#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Package to use a "Phenology Available" image collection in Google Earth Engine."""

__all__ = (
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
)

__title__ = "GEE phenology"
__summary__ = "Generate a 'Phenology Available Result' image in Google Earth Engine"
__uri__ = "https://github.com/francbartoli/gee_pheno"

__author__ = """Francesco Bartoli"""
__email__ = 'francesco.bartoli@geobeyond.it'

__version__ = '0.1.5'

__license__ = "GNU GENERAL PUBLIC LICENSE, Version 3"
__copyright__ = "Francesco Bartoli"
